@@ -19,15 +19,14 @@
 #include <panel-applet.h>
 #include <panel-applet-gconf.h>
 
-#include <gtk/gtk.h>
-#include <libbonobo.h>
-#include <libgnome/libgnome.h>
-#include <libgnomeui/libgnomeui.h>
+#include <gtk/gtkdialog.h>
+#include <libgnomeui/gnome-help.h>
 #include <libgnome/gnome-desktop-item.h>
 #include <glade/glade-xml.h>
 #include <libwnck/libwnck.h>
 #include <gconf/gconf-client.h>
 
+#include "wncklet.h"
 #include "window-list.h"
 
 typedef struct {
@@ -44,7 +43,7 @@ typedef struct {
 	int size;
 	gint maximum_size;
 
-        GnomeIconTheme *icon_theme;
+        GtkIconTheme *icon_theme;
   
 	/* Properties: */
 	GtkWidget *properties_dialog;
@@ -129,28 +128,17 @@ response_cb (GtkWidget    *widget,
 		gtk_widget_hide (widget);
 }
 
-static WnckScreen *
-applet_get_screen (GtkWidget *applet)
-{
-	int screen_num;
-
-	if (!gtk_widget_has_screen (applet))
-		return wnck_screen_get_default ();
-
-	screen_num = gdk_screen_get_number (gtk_widget_get_screen (applet));
-
-	return wnck_screen_get (screen_num);
-}
-
 static void
 applet_realized (PanelApplet  *applet,
 		 TasklistData *tasklist)
 {
 	WnckScreen *screen;
 
-	screen = applet_get_screen (GTK_WIDGET (applet));
+	screen = wncklet_get_screen (GTK_WIDGET (applet));
 
 	wnck_tasklist_set_screen (WNCK_TASKLIST (tasklist->tasklist), screen);
+
+	tasklist->icon_theme = gtk_icon_theme_get_for_screen (gtk_widget_get_screen (tasklist->applet));
 }
 
 static void
@@ -201,26 +189,13 @@ static void
 applet_change_background (PanelApplet               *applet,
 			  PanelAppletBackgroundType  type,
 			  GdkColor                  *color,
-			  const gchar               *pixmap,
+			  GdkPixmap                 *pixmap,
 			  TasklistData              *tasklist)
 {
-	if (type == PANEL_NO_BACKGROUND) {
-		GtkRcStyle *rc_style = gtk_rc_style_new ();
-
-		gtk_widget_modify_style (tasklist->applet, rc_style);
-
-		g_object_unref (rc_style);
-	}
-	else if (type == PANEL_COLOR_BACKGROUND) {
-		gtk_widget_modify_bg (tasklist->applet,
-				      GTK_STATE_NORMAL,
-				      color);
-	} else { /* pixmap */
-		/* FIXME: Handle this when the panel support works again */
-	}
+	wncklet_change_background (GTK_WIDGET (tasklist->applet), type,
+				   color, pixmap);
 }
 
-
 static void
 applet_change_pixel_size (PanelApplet  *applet,
 			  gint          size,
@@ -239,9 +214,6 @@ destroy_tasklist(GtkWidget * widget, TasklistData *tasklist)
 {
 	GConfClient *client = gconf_client_get_default ();
 
-        g_object_unref (tasklist->icon_theme);
-        tasklist->icon_theme = NULL;
-        
 	gconf_client_notify_remove (client, tasklist->listeners[0]);
 	gconf_client_notify_remove (client, tasklist->listeners[1]);
 	gconf_client_notify_remove (client, tasklist->listeners[2]);
@@ -583,22 +555,11 @@ icon_loader_func (const char  *icon,
                   void        *data)
 {
         TasklistData *tasklist;
-        char *file;
-        GdkPixbuf *pixbuf;
         
         tasklist = data;
-        
-        file = gnome_desktop_item_find_icon (tasklist->icon_theme,
-                                             icon, size, 0);
 
-        if (file == NULL)
-                return NULL;
-
-        pixbuf = gdk_pixbuf_new_from_file (file, NULL);
-
-        g_free (file);
-
-        return pixbuf;
+	return gtk_icon_theme_load_icon (tasklist->icon_theme,
+					 icon, size, 0, NULL);
 }
 
 gboolean
@@ -623,8 +584,6 @@ window_list_applet_fill (PanelApplet *applet)
 
 	setup_gconf (tasklist);
 
-        tasklist->icon_theme = gnome_icon_theme_new ();
-        
 	error = NULL;
 	tasklist->include_all_workspaces = panel_applet_gconf_get_bool (applet, "display_all_workspaces", &error);
 	if (error) {
@@ -664,7 +623,7 @@ window_list_applet_fill (PanelApplet *applet)
 		break;
 	}
 
-	tasklist->screen = applet_get_screen (tasklist->applet);
+	tasklist->screen = wncklet_get_screen (tasklist->applet);
 
 	/* because the tasklist doesn't respond to signals at the moment */
 	wnck_screen_force_update (tasklist->screen);
@@ -724,7 +683,6 @@ window_list_applet_fill (PanelApplet *applet)
 			  "change_size",
 			  G_CALLBACK (applet_change_pixel_size),
 			  tasklist);
-	/* FIXME: initial background, this needs some panel-applet voodoo */
 	g_signal_connect (G_OBJECT (tasklist->applet),
 			  "change_background",
 			  G_CALLBACK (applet_change_background),
@@ -757,31 +715,8 @@ display_help_dialog (BonoboUIComponent *uic,
 		     TasklistData      *tasklist,
 		     const gchar       *verbname)
 {
-	GError *error = NULL;
-
-	gnome_help_display_desktop_on_screen (
-			NULL, "window-list", "window-list", NULL,
-			gtk_widget_get_screen (tasklist->applet),
-			&error);
-	if (error) {
-		GtkWidget *dialog;
-		dialog = gtk_message_dialog_new (NULL,
-						 GTK_DIALOG_MODAL,
-						 GTK_MESSAGE_ERROR,
-						 GTK_BUTTONS_OK,
-						  _("There was an error displaying help: %s"),
-						 error->message);
-
-		g_signal_connect (dialog, "response",
-				  G_CALLBACK (gtk_widget_destroy),
-				  NULL);
-
-		gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
-		gtk_window_set_screen (GTK_WINDOW (dialog),
-				       gtk_widget_get_screen (tasklist->applet));
-		gtk_widget_show (dialog);
-		g_error_free (error);
-	}
+	wncklet_display_help (tasklist->applet, "window-list",
+			      "window-list", NULL);
 }
 
 static void
@@ -789,9 +724,6 @@ display_about_dialog (BonoboUIComponent *uic,
 		      TasklistData      *tasklist,
 		      const gchar       *verbname)
 {
-	GdkPixbuf *pixbuf = NULL;
-	gchar *file;
-	
 	static const gchar *authors[] =
 	{
 		"Alexander Larsson <alla@lysator.liu.se>",
@@ -801,43 +733,18 @@ display_about_dialog (BonoboUIComponent *uic,
 		"Sun GNOME Documentation Team <gdocteam@sun.com>",
 		NULL
 	};
-	const char *translator_credits = _("translator_credits");
+	const char *translator_credits = _("translator-credits");
 
-	if (tasklist->about) {
-		gtk_window_set_screen (GTK_WINDOW (tasklist->about),
-				       gtk_widget_get_screen (tasklist->applet));
-		gtk_widget_show (tasklist->about);
-		gtk_window_present (GTK_WINDOW (tasklist->about));
-		return;
-	}
-
-	file = gnome_icon_theme_lookup_icon (tasklist->icon_theme,
-					     "panel-window-list",
-					     48, NULL, NULL);
-	pixbuf = gdk_pixbuf_new_from_file (file, NULL);
-	g_free(file);
-
-	tasklist->about = gnome_about_new (_("Window List"), VERSION,
-				 "Copyright \xc2\xa9 2001-2002 Red Hat, Inc.",
-				 _("The Window List shows a list of all windows and lets you browse them."),
-				 authors,
-				 documenters,
-				 strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
-				 pixbuf);
-	
-	gtk_window_set_wmclass (GTK_WINDOW (tasklist->about), "tasklist", "Tasklist");
-	gtk_window_set_screen (GTK_WINDOW (tasklist->about),
-			       gtk_widget_get_screen (tasklist->applet));
-
-	if (pixbuf) {
-		gtk_window_set_icon (GTK_WINDOW (tasklist->about), pixbuf);
-		g_object_unref (pixbuf);
-	}
-	
-	g_signal_connect (G_OBJECT(tasklist->about), "destroy",
-			  (GCallback)gtk_widget_destroyed, &tasklist->about);
-	
-	gtk_widget_show (tasklist->about);
+	wncklet_display_about (tasklist->applet, &tasklist->about,
+			       _("Window List"),
+			       "Copyright \xc2\xa9 2001-2002 Red Hat, Inc.",
+			       _("The Window List shows a list of all windows and lets you browse them."),
+			       authors,
+			       documenters,
+			       translator_credits,
+			       "panel-window-list",
+			       "tasklist",
+			       "Tasklist");
 }
 
 static void
@@ -1063,8 +970,6 @@ display_properties_dialog (BonoboUIComponent *uic,
 			   TasklistData      *tasklist,
 			   const gchar       *verbname)
 {
-	gchar *window_icon;
-
 	if (tasklist->properties_dialog == NULL) {
 		GladeXML  *xml;
 
@@ -1079,13 +984,8 @@ display_properties_dialog (BonoboUIComponent *uic,
 		g_object_unref (G_OBJECT (xml));
 	}
 
-	window_icon = gnome_icon_theme_lookup_icon (tasklist->icon_theme,
-						    "panel-window-list",
-						    48, NULL, NULL);
-	if (window_icon) {
-		gnome_window_icon_set_from_file (GTK_WINDOW (tasklist->properties_dialog), window_icon);
-		g_free (window_icon);
-	}
+	gtk_window_set_icon_name (GTK_WINDOW (tasklist->properties_dialog),
+				  "panel-window-list"); 
 
 	gtk_window_set_resizable (GTK_WINDOW (tasklist->properties_dialog), FALSE);
 	gtk_window_set_screen (GTK_WINDOW (tasklist->properties_dialog),
