@@ -12,13 +12,12 @@
 #include <signal.h>
 #include <sys/wait.h>
 
-#include <libgnome/libgnome.h>
-#include <libgnomeui/libgnomeui.h>
+#include <libgnomeui/gnome-authentication-manager.h>
+#include <libgnomeui/gnome-ui-init.h>
 
 #include <libbonoboui.h>
 #include <bonobo-activation/bonobo-activation.h>
 
-#include "menu-fentry.h"
 #include "menu.h"
 #include "panel.h"
 #include "panel-util.h"
@@ -41,12 +40,7 @@ GSList *panel_list = NULL;
 
 GtkTooltips *panel_tooltips = NULL;
 
-GnomeIconTheme *panel_icon_theme = NULL;
-
-static char *profile_arg;
-
 static const struct poptOption options[] = {
-  {"profile", '\0', POPT_ARG_STRING, &profile_arg, 0, N_("Specify a profile name to load"), NULL},
   {NULL, '\0', 0, NULL, 0}
 };
 
@@ -67,10 +61,9 @@ main (int argc, char **argv)
 	if (!panel_shell_register ())
 		return -1;
 
-	panel_icon_theme = gnome_icon_theme_new ();
-	gnome_icon_theme_set_allow_svg (panel_icon_theme, TRUE);
-	
-	panel_register_window_icon ();
+	gnome_authentication_manager_init ();
+
+	gtk_window_set_default_icon_name ("gnome-panel");
 
 	panel_tooltips = gtk_tooltips_new ();
 
@@ -78,8 +71,6 @@ main (int argc, char **argv)
 	panel_multiscreen_init ();
 	panel_init_stock_icons_and_items ();
 
-        init_menus ();
-
 	gconf_client_add_dir (panel_gconf_get_client (),
 			      "/desktop/gnome/interface",
 			      GCONF_CLIENT_PRELOAD_NONE,
@@ -87,7 +78,7 @@ main (int argc, char **argv)
 
 	panel_global_config_load ();
 	panel_lockdown_init ();
-	panel_profile_load (profile_arg);
+	panel_profile_load ();
 
 	panel_session_init (argv [0]);
 
