@@ -32,31 +32,26 @@
 #include <gconf/gconf-client.h>
 
 #include "launcher.h"
-#include "nothing.h"
 #include "panel-util.h"
-#include "panel-gconf.h"
 #include "panel.h"
 #include "drawer.h"
 #include "panel-config-global.h"
 #include "panel-stock-icons.h"
 #include "panel-action-button.h"
 #include "panel-recent.h"
-#include "panel-multiscreen.h"
-#include "panel-toplevel.h"
 #include "panel-profile.h"
 #include "panel-menu-button.h"
 #include "panel-globals.h"
-#include "panel-properties-dialog.h"
 #include "panel-run-dialog.h"
 #include "panel-lockdown.h"
-#include "panel-addto.h"
 
 typedef struct {
-	GtkWidget   *pixmap;
-	const char  *stock_id;
-	char        *image;
-	char        *fallback_image;
-	GtkIconSize  icon_size;
+	GtkWidget    *pixmap;
+	const char   *stock_id;
+	char         *image;
+	char         *fallback_image;
+	GtkIconTheme *icon_theme;
+	GtkIconSize   icon_size;
 } IconToLoad;
 
 typedef struct {
@@ -84,8 +79,6 @@ static void panel_load_menu_image_deferred (GtkWidget   *image_menu_item,
 
 static gboolean panel_menu_key_press_handler (GtkWidget   *widget,
 					      GdkEventKey *event);
-static PanelWidget *menu_get_panel     (GtkWidget *menu);
-static GdkScreen   *menuitem_to_screen (GtkWidget *menuitem);
 
 static inline gboolean
 panel_menu_have_icons (void)
@@ -96,7 +89,7 @@ panel_menu_have_icons (void)
 			NULL);
 }
 
-static GtkWidget *
+GtkWidget *
 add_menu_separator (GtkWidget *menu)
 {
 	GtkWidget *menuitem;
@@ -109,114 +102,6 @@ add_menu_separator (GtkWidget *menu)
 	return menuitem;
 }
 
-static gboolean
-check_for_screen (GtkWidget *w, GdkEvent *ev, gpointer data)
-{
-	static int times = 0;
-	if (ev->type != GDK_KEY_PRESS)
-		return FALSE;
-	if (ev->key.keyval == GDK_f ||
-	    ev->key.keyval == GDK_F) {
-		times++;
-		if (times == 3) {
-			times = 0;
-			start_screen_check ();
-		}
-	}
-	return FALSE;
-}
-
-static void
-show_about_dialog (GtkWidget *menuitem)
-{
-	static GtkWidget *about = NULL;
-	char             *authors [] = {
-		"Alex Larsson (alexl@redhat.com)",
-		"Anders Carlsson (andersca@gnu.org)",
-		"Arvind Samptur (arvind.samptur@wipro.com)",
-		"Darin Adler (darin@bentspoon.com)",
-		"Elliot Lee (sopwith@redhat.com)",
-		"Federico Mena (quartic@gimp.org)",
-		"George Lebl (jirka@5z.com)",
-		"Glynn Foster (glynn.foster@sun.com)",
-		"Ian Main (imain@gtk.org)",
-		"Ian McKellar <yakk@yakk.net>",
-		"Jacob Berkman (jberkman@andrew.cmu.edu)",
-		"Mark McLoughlin (mark@skynet.ie)",
-		"Martin Baulig (baulig@suse.de)",
-		"Miguel de Icaza (miguel@kernel.org)",
-		"Owen Taylor (otaylor@redhat.com)",
-		"Padraig O'Briain (padraig.obriain@sun.com)",
-		"Seth Nickell (snickell@stanford.edu)",
-		"Stephen Browne (stephen.browne@sun.com)",
-		"Tom Tromey (tromey@cygnus.com)",
-		"Vincent Untz (vincent@vuntz.net)",
-		N_("And many, many others..."),
-		NULL
-	};
-	char *documenters[] = {
-	        "Alexander Kirillov (kirillov@math.sunysb.edu)",
-		"Dan Mueth (d-mueth@uchicago.edu)",
-		"Dave Mason (dcm@redhat.com)",
-		NULL
-	  };
-	/* Translator credits */
-	char *translator_credits = _("translator-credits");
-	int   i;
-
-	if (about) {
-		gtk_window_set_screen (
-			GTK_WINDOW (about), menuitem_to_screen (menuitem));
-		gtk_window_present (GTK_WINDOW (about));
-		return;
-	}
-
-	for (i = 0; authors [i]; i++)
-		authors [i] = _(authors [i]);
-
-	about = gtk_about_dialog_new ();
-	g_object_set (about,
-		      "name",  _("The GNOME Panel"),
-		      "version", VERSION,
-		      "copyright", "Copyright \xc2\xa9 1997-2003 Free Software Foundation, Inc.",
-		      "comments", _("This program is responsible for launching other "
-				    "applications and embedding small applets within itself."),
-		      "authors", authors,
-		      "documenters", documenters,
-		      "translator_credits", strcmp (translator_credits, "translator-credits") != 0 ? translator_credits : NULL,
-		      "logo_icon_name", "gnome-panel",
-		      NULL);
-
-	gtk_window_set_wmclass (GTK_WINDOW (about), "about_dialog", "Panel");
-	gtk_window_set_screen (GTK_WINDOW (about),
-			       menuitem_to_screen (menuitem));
-	g_signal_connect (about, "destroy",
-			  G_CALLBACK (gtk_widget_destroyed),
-			  &about);
-	g_signal_connect (about, "event",
-			  G_CALLBACK (check_for_screen), NULL);
-
-	gtk_widget_show (about);
-}
-
-static void
-about_gnome_cb (GtkWidget *menuitem,
-		char      *program_path)
-{
-	GdkScreen *screen = menuitem_to_screen (menuitem);
-	GError    *error = NULL;
-
-	if (!gdk_spawn_command_line_on_screen (screen, program_path, &error)) {
-		panel_error_dialog (screen,
-				    "cannot_exec_about_gnome",
-				    _("Cannot execute '%s'"),
-				    _("%s: %s"),
-				    _("About GNOME"),
-				    program_path, error->message);
-		g_error_free (error);
-	}
-}
-
 static void
 activate_app_def (GtkWidget     *menuitem,
 		  MenuTreeEntry *entry)
@@ -236,7 +121,7 @@ activate_app_def (GtkWidget     *menuitem,
 			item, NULL, 0, menuitem_to_screen (menuitem), &error);
 		if (error) {
 			panel_error_dialog (menuitem_to_screen (menuitem),
-					    "cannot_launch_entry",
+					    "cannot_launch_entry", TRUE,
 					    _("Cannot launch entry"),
 					    "%s",
 					    error->message);
@@ -248,7 +133,7 @@ activate_app_def (GtkWidget     *menuitem,
 		g_assert (error != NULL);
 
 		panel_error_dialog (menuitem_to_screen (menuitem),
-				    "cannot_load_entry",
+				    "cannot_load_entry", TRUE,
 				    _("Cannot load entry"),
 				    "%s",
 				    error->message);
@@ -256,7 +141,7 @@ activate_app_def (GtkWidget     *menuitem,
 	}
 }
 
-static PanelWidget *
+PanelWidget *
 menu_get_panel (GtkWidget *menu)
 {
 	PanelWidget *retval = NULL;
@@ -304,7 +189,7 @@ setup_menu_panel (GtkWidget *menu)
 	g_object_set_data (G_OBJECT (menu), "menu_panel", panel);
 }
 
-static GdkScreen *
+GdkScreen *
 menuitem_to_screen (GtkWidget *menuitem)
 {
 	PanelWidget *panel_widget;
@@ -401,7 +286,7 @@ panel_create_menu (void)
 	if (!registered_icon_theme_changer) {
 		registered_icon_theme_changer = TRUE;
 
-		g_signal_connect (panel_icon_theme, "changed",
+		g_signal_connect (gtk_icon_theme_get_default (), "changed",
 				  G_CALLBACK (icon_theme_changed), NULL);
 	}
 	
@@ -474,10 +359,11 @@ remove_pixmap_from_loaded (gpointer data, GObject *where_the_object_was)
 }
 
 GdkPixbuf *
-panel_make_menu_icon (const char *icon,
-		      const char *fallback,
-		      int         size,
-		      gboolean   *long_operation)
+panel_make_menu_icon (GtkIconTheme *icon_theme,
+		      const char   *icon,
+		      const char   *fallback,
+		      int           size,
+		      gboolean     *long_operation)
 {
 	GdkPixbuf *pb;
 	char *file, *key;
@@ -487,15 +373,9 @@ panel_make_menu_icon (const char *icon,
 
 	file = NULL;
 	if (icon != NULL)
-		file = gnome_desktop_item_find_icon (panel_icon_theme,
-						     icon,
-						     size /* desired size */,
-						     0 /* flags */);
+		file = panel_find_icon (icon_theme, icon, size);
 	if (file == NULL && fallback != NULL)
-		file = gnome_desktop_item_find_icon (panel_icon_theme,
-						     fallback,
-						     size /* desired size */,
-						     0 /* flags */);
+		file = panel_find_icon (icon_theme, fallback, size);
 
 	if (file == NULL)
 		return NULL;
@@ -529,10 +409,26 @@ panel_make_menu_icon (const char *icon,
 	}
 
 	if (loaded &&
-	    (gdk_pixbuf_get_width (pb) != size ||
+	    (gdk_pixbuf_get_width (pb) != size &&
 	     gdk_pixbuf_get_height (pb) != size)) {
 		GdkPixbuf *pb2;
-		pb2 = gdk_pixbuf_scale_simple (pb, size, size,
+		int        dest_width;
+		int        dest_height;
+		int        width;
+		int        height;
+
+		width  = gdk_pixbuf_get_width (pb);
+		height = gdk_pixbuf_get_height (pb);
+
+		if (height > width) {
+			dest_width  = (size * width) / height;
+			dest_height = size;
+		} else {
+			dest_width  = size;
+			dest_height = (size * height) / width;
+		}
+
+		pb2 = gdk_pixbuf_scale_simple (pb, dest_width, dest_height,
 					       GDK_INTERP_BILINEAR);
 		g_object_unref (G_OBJECT (pb));
 		pb = pb2;
@@ -671,7 +567,8 @@ load_icons_handler_again:
 
 		gtk_icon_size_lookup (icon->icon_size, NULL, &icon_height);
 
-		pb = panel_make_menu_icon (icon->image,
+		pb = panel_make_menu_icon (icon->icon_theme,
+					   icon->image,
 					   icon->fallback_image,
 					   icon_height,
 					   &long_operation);
@@ -796,6 +693,8 @@ add_menudrawer_to_panel (GtkWidget     *menuitem,
 	add_drawers_from_dir (directory,
 			      insertion_pos,
 			      panel_profile_get_toplevel_id (panel->toplevel));
+
+	menu_tree_directory_unref (directory);
 }
 
 static void
@@ -807,6 +706,16 @@ add_menu_to_panel (GtkWidget     *menuitem,
 	PanelData         *pd;
 	int                insertion_pos;
 	char              *menu_path;
+	char              *menu_filename;
+	GtkWidget         *parent;
+
+	menu_filename = g_object_get_data (G_OBJECT (menuitem->parent),
+					   "panel-menu-tree-filename");
+
+	if (!menu_filename) {
+		g_warning ("Cannot find the filename for the menu");
+		return;
+	}
 
 	directory = menu_tree_entry_get_parent (entry);
 
@@ -819,11 +728,14 @@ add_menu_to_panel (GtkWidget     *menuitem,
 
 	panel_menu_button_create (panel->toplevel,
 				  insertion_pos,
+				  menu_filename,
 				  menu_path,
 				  TRUE,
 				  menu_tree_directory_get_name (directory));
 
 	g_free (menu_path);
+
+	menu_tree_directory_unref (directory);
 }
 
 /*most of this function stolen from the real gtk_menu_popup*/
@@ -883,16 +795,31 @@ create_item_context_menu (GtkWidget   *item,
 			  PanelWidget *panel_widget)
 {
 	MenuTreeEntry *entry;
+	GtkWidget     *parent;
 	GtkWidget     *menu;
 	GtkWidget     *submenu;
 	GtkWidget     *menuitem;
+	char          *menu_filename;
 	gboolean       id_lists_writable;
 
 	id_lists_writable = panel_profile_id_lists_are_writable ();
 
 	entry = g_object_get_data (G_OBJECT (item), "panel-menu-tree-entry");
-	g_assert (entry != NULL);
+	if (!entry)
+		return NULL;
 
+	parent = item->parent;
+
+	while (parent) {
+		menu_filename = g_object_get_data (G_OBJECT (parent),
+						   "panel-menu-tree-filename");
+		if (menu_filename)
+			break;
+
+		parent = gtk_menu_get_attach_widget (GTK_MENU (parent))->parent;
+		if (!GTK_IS_MENU (parent))
+			break;
+	}
 
 	menu = create_empty_menu ();
 	g_object_set_data_full (G_OBJECT (item),
@@ -902,8 +829,6 @@ create_item_context_menu (GtkWidget   *item,
 	gtk_object_sink (GTK_OBJECT (menu));
 
 	g_object_set_data (G_OBJECT (menu), "menu_panel", panel_widget);
-
-
 	g_signal_connect (menu, "deactivate",
 			  G_CALLBACK (restore_grabs), item);
 
@@ -922,7 +847,10 @@ create_item_context_menu (GtkWidget   *item,
 	submenu = create_empty_menu ();
 
 	g_object_set_data (G_OBJECT (submenu), "menu_panel", panel_widget);
-
+	g_object_set_data_full (G_OBJECT (submenu),
+				"panel-menu-tree-filename",
+				g_strdup (menu_filename),
+				(GDestroyNotify) g_free);
 
 	menuitem = gtk_image_menu_item_new ();
 	setup_menuitem (menuitem,
@@ -977,6 +905,9 @@ show_item_menu (GtkWidget      *item,
 	if (!menu)
 		menu = create_item_context_menu (item, panel_widget);
 
+	if (!menu)
+		return FALSE;
+
 	gtk_menu_set_screen (GTK_MENU (menu),
 			     gtk_window_get_screen (GTK_WINDOW (panel_widget->toplevel)));
 
@@ -1202,23 +1133,59 @@ drag_data_get_string_cb (GtkWidget *widget, GdkDragContext     *context,
 }
 
 void
-setup_internal_applet_drag (GtkWidget *menuitem, const char *applet_type)
+setup_uri_drag (GtkWidget  *menuitem,
+		const char *uri,
+		const char *icon)
 {
 	static GtkTargetEntry menu_item_targets[] = {
-		{ "application/x-panel-applet-internal", 0, 0 }
+		{ "text/uri-list", 0, 0 }
 	};
 
-	if (!applet_type || panel_lockdown_get_locked_down ())
+	if (panel_lockdown_get_locked_down ())
 		return;
 
 	gtk_drag_source_set (menuitem,
 			     GDK_BUTTON1_MASK|GDK_BUTTON2_MASK,
 			     menu_item_targets, 1,
 			     GDK_ACTION_COPY);
+
+	/* FIXME: waiting for bug #116577
+	gtk_drag_source_set_icon_name (GTK_WIDGET (button), icon); */
 	
 	g_signal_connect_data (G_OBJECT (menuitem), "drag_data_get",
 			       G_CALLBACK (drag_data_get_string_cb),
-			       g_strdup (applet_type),
+			       g_strdup (uri),
+			       (GClosureNotify)g_free,
+			       0 /* connect_flags */);
+	g_signal_connect (G_OBJECT (menuitem), "drag_end",
+			  G_CALLBACK (drag_end_menu_cb), NULL);
+}
+
+void
+setup_internal_applet_drag (GtkWidget             *menuitem,
+			    PanelActionButtonType  type)
+{
+	static GtkTargetEntry menu_item_targets[] = {
+		{ "application/x-panel-applet-internal", 0, 0 }
+	};
+
+	if (panel_lockdown_get_locked_down ())
+		return;
+
+	gtk_drag_source_set (menuitem,
+			     GDK_BUTTON1_MASK|GDK_BUTTON2_MASK,
+			     menu_item_targets, 1,
+			     GDK_ACTION_COPY);
+
+	/* FIXME: waiting for bug #116577
+	if (panel_action_get_icon_name (type)  != NULL)
+		gtk_drag_source_set_icon_name (GTK_WIDGET (button),
+					panel_action_get_icon_name (type);
+					*/
+	
+	g_signal_connect_data (G_OBJECT (menuitem), "drag_data_get",
+			       G_CALLBACK (drag_data_get_string_cb),
+			       g_strdup (panel_action_get_drag_id (type)),
 			       (GClosureNotify)g_free,
 			       0 /* connect_flags */);
 	g_signal_connect (G_OBJECT (menuitem), "drag_end",
@@ -1228,16 +1195,29 @@ setup_internal_applet_drag (GtkWidget *menuitem, const char *applet_type)
 static void
 submenu_to_display (GtkWidget *menu)
 {
-	if (g_object_get_data (G_OBJECT (menu), "panel-menu-needs-loading")) {
-		MenuTreeDirectory *directory;
+	MenuTree          *tree;
+	MenuTreeDirectory *directory;
+	const char        *menu_path;
 
-		directory = g_object_get_data (G_OBJECT (menu), "panel-menu-tree-directory");
+	if (!g_object_get_data (G_OBJECT (menu), "panel-menu-needs-loading"))
+		return;
 
-		if (directory)
-			populate_menu_from_directory (menu, directory);
+	g_object_set_data (G_OBJECT (menu), "panel-menu-needs-loading", NULL);
 
-		g_object_set_data (G_OBJECT (menu), "panel-menu-needs-loading", NULL);
+	if (!(directory = g_object_get_data (G_OBJECT (menu), "panel-menu-tree-directory")) &&
+	     (menu_path = g_object_get_data (G_OBJECT (menu), "panel-menu-tree-path"))) {
+		if (!(tree = g_object_get_data (G_OBJECT (menu), "panel-menu-tree")))
+			return;
+
+		directory = menu_tree_get_directory_from_path (tree, menu_path);
+
+		g_object_set_data_full (G_OBJECT (menu), "panel-menu-tree-directory",
+					directory,
+					(GDestroyNotify) menu_tree_directory_unref);
 	}
+
+	if (directory)
+		populate_menu_from_directory (menu, directory);
 }
 
 static GtkWidget *
@@ -1256,6 +1236,10 @@ create_fake_menu (MenuTreeDirectory *directory)
 			   "panel-menu-needs-loading",
 			   GUINT_TO_POINTER (TRUE));
 
+	g_object_set_data (G_OBJECT (menu),
+			   "panel-menu-needs-appending",
+			   GUINT_TO_POINTER (TRUE));
+
 	g_signal_connect (menu, "show",
 			  G_CALLBACK (submenu_to_display), NULL);
 	g_signal_connect (menu, "button_press_event",
@@ -1346,6 +1330,10 @@ create_menuitem (GtkWidget     *menu,
 				     menu_item_targets, 1,
 				     GDK_ACTION_COPY);
 
+		/* FIXME: waiting for bug #116577
+		gtk_drag_source_set_icon_name (GTK_WIDGET (button),
+					menu_tree_entry_get_icon (entry));
+		*/
 		g_signal_connect (menuitem, "drag_data_get",
 				  G_CALLBACK (drag_data_get_menu_cb), entry);
 		g_signal_connect (menuitem, "drag_end",
@@ -1367,12 +1355,15 @@ handle_menu_tree_changed (MenuTree  *tree,
 
 	g_object_set_data_full (G_OBJECT (menu),
 				"panel-menu-tree-directory",
-				menu_tree_get_root_directory (tree),
-				(GDestroyNotify) menu_tree_directory_unref);
+				NULL, NULL);
 
 	g_object_set_data (G_OBJECT (menu),
 			   "panel-menu-needs-loading",
 			   GUINT_TO_POINTER (TRUE));
+
+	g_object_set_data (G_OBJECT (menu),
+			   "panel-menu-needs-appending",
+			   GUINT_TO_POINTER (TRUE));
 }
 
 static void
@@ -1388,30 +1379,41 @@ GtkWidget *
 create_applications_menu (const char *menu_file,
 			  const char *menu_path)
 {
-	MenuTree          *tree;
-	MenuTreeDirectory *root;
-	GtkWidget         *menu;
+	MenuTree  *tree;
+	GtkWidget *menu;
+
+	menu = create_empty_menu ();
 
 	tree = menu_tree_lookup (menu_file);
 
-	if (!menu_path)
-		root = menu_tree_get_root_directory (tree);
-	else
-		root = menu_tree_get_directory_from_path (tree, menu_path);
-
-	if (root) {
-		menu = create_fake_menu (root);
-
-		menu_tree_directory_unref (root);
-	} else {
-		menu = create_empty_menu ();
-	}
+	g_object_set_data_full (G_OBJECT (menu),
+				"panel-menu-tree-filename",
+				g_strdup (menu_file),
+				(GDestroyNotify) g_free);
 
 	g_object_set_data_full (G_OBJECT (menu),
 				"panel-menu-tree",
 				menu_tree_ref (tree),
 				(GDestroyNotify) menu_tree_unref);
 
+	g_object_set_data_full (G_OBJECT (menu),
+				"panel-menu-tree-path",
+				g_strdup (menu_path ? menu_path : "/"),
+				(GDestroyNotify) g_free);
+	
+	g_object_set_data (G_OBJECT (menu),
+			   "panel-menu-needs-loading",
+			   GUINT_TO_POINTER (TRUE));
+
+	g_object_set_data (G_OBJECT (menu),
+			   "panel-menu-needs-appending",
+			   GUINT_TO_POINTER (TRUE));
+
+	g_signal_connect (menu, "show",
+			  G_CALLBACK (submenu_to_display), NULL);
+	g_signal_connect (menu, "button_press_event",
+			  G_CALLBACK (menu_dummy_button_press_event), NULL);
+
 	menu_tree_add_monitor (tree,
 			       (MenuTreeChangedFunc) handle_menu_tree_changed,
 			       menu);
@@ -1467,269 +1469,49 @@ populate_menu_from_directory (GtkWidget         *menu,
 	return menu;
 }
 
-static void
-create_new_panel (GtkWidget *menuitem)
-{
-	panel_profile_create_toplevel (gtk_widget_get_screen (menuitem));
-}
-
 void
-setup_stock_menu_item (GtkWidget   *item,
-		       GtkIconSize  icon_size,
-		       const char  *stock_id,
-		       const char  *title,
-		       gboolean     invisible_mnemonic)
+setup_menu_item_with_icon (GtkWidget   *item,
+			   GtkIconSize  icon_size,
+			   const char  *icon_name,
+			   const char  *stock_id,
+			   const char  *title,
+			   gboolean     invisible_mnemonic)
 {
-	if (stock_id)
-		panel_load_menu_image_deferred (item, icon_size, stock_id, NULL, NULL);
+	if (icon_name || stock_id)
+		panel_load_menu_image_deferred (item, icon_size,
+						stock_id, icon_name, NULL);
 
 	setup_menuitem (item, icon_size, NULL, title, invisible_mnemonic);
 }
-	  
-static void
-remove_panel (GtkWidget *menuitem,
-              gpointer data)
-{
-	PanelWidget   *panel_widget;
-	PanelToplevel *toplevel;
-
-	panel_widget = menu_get_panel (menuitem);
-	toplevel     = panel_widget->toplevel;
-
-	if (panel_toplevel_is_last_unattached (toplevel)) {
-		panel_error_dialog (menuitem_to_screen (menuitem),
-				    "cannot_remove_last_panel",
-				    _("You cannot remove your last panel."),
-				    NULL);
-		return;
-	}
-
-        panel_delete (toplevel);
-}
-
-static void
-setup_remove_this_panel (GtkWidget *menu,
-			 GtkWidget *menuitem)
-{
-	PanelWidget *panel_widget;
-	GtkWidget   *label;
-	gboolean     sensitive;
-
-	panel_widget = menu_get_panel (menu);
-
-	g_assert (PANEL_IS_TOPLEVEL (panel_widget->toplevel));
-
-	sensitive =
-		panel_toplevel_is_last_unattached (panel_widget->toplevel) ||
-		!panel_lockdown_get_locked_down () ||
-		!panel_profile_id_lists_are_writable ();
-
-	gtk_widget_set_sensitive (menuitem, sensitive);
-
-	label = GTK_BIN(menuitem)->child;
-	if(GTK_IS_BOX(label)) {
-		GList *li, *list;
-		list = gtk_container_get_children(GTK_CONTAINER(label));
-		for(li = list; li; li = li->next) {
-			if(GTK_IS_LABEL(li->data)) {
-				label = li->data;
-				break;
-			}
-		}
-		g_list_free(list);
-	}
-	if(!GTK_IS_LABEL(label)) {
-		g_warning("We can't find the label of a menu item");
-		return;
-	}
-
-	/* this will not handle the case of menu being torn off
-	 * and then the confirm_panel_remove changed, but oh well */
-	if (panel_widget->applet_list &&
-	    panel_global_config_get_confirm_panel_remove ())
-		gtk_label_set_text_with_mnemonic (
-			GTK_LABEL (label), _("_Delete This Panel..."));
-	else
-		gtk_label_set_text_with_mnemonic (
-			GTK_LABEL (label), _("_Delete This Panel"));
-}
-
-static void
-make_panel_submenu (PanelWidget *panel_widget,
-		    GtkWidget   *menu)
-{
-	GtkWidget             *menuitem;
-
-	menuitem = gtk_image_menu_item_new ();
-	setup_menuitem (menuitem,
-			GTK_ICON_SIZE_MENU,
-			gtk_image_new_from_stock (GTK_STOCK_ADD, GTK_ICON_SIZE_MENU),
-			_("_Add to Panel..."),
-			FALSE);
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
-
-        g_signal_connect (G_OBJECT (menuitem), "activate",
-	      	       	  G_CALLBACK (panel_addto_present), panel_widget);
-
-	if (!panel_profile_id_lists_are_writable ())
-		gtk_widget_set_sensitive (menuitem, FALSE);
-
-	menuitem = gtk_image_menu_item_new ();
-
-	setup_menuitem (menuitem, 
-			GTK_ICON_SIZE_MENU,
-			gtk_image_new_from_stock (GTK_STOCK_DELETE, GTK_ICON_SIZE_MENU),
-			_("_Delete This Panel"),
-			FALSE);
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
-	g_signal_connect (G_OBJECT (menuitem), "activate",
-			  G_CALLBACK (remove_panel),
-			  NULL);
-	g_signal_connect (G_OBJECT (menu), "show",
-			  G_CALLBACK(setup_remove_this_panel),
-			  menuitem);
-
-	menuitem = gtk_image_menu_item_new ();
-	setup_menuitem (menuitem,
-			GTK_ICON_SIZE_MENU,
-			gtk_image_new_from_stock (
-						  GTK_STOCK_PROPERTIES, GTK_ICON_SIZE_MENU),
-			_("_Properties"),
-			FALSE);
-
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
-	g_signal_connect_swapped (menuitem, "activate",
-				  G_CALLBACK (panel_properties_dialog_present), 
-				  panel_widget->toplevel);
-
-	add_menu_separator (menu);
-
-	menuitem = gtk_image_menu_item_new ();
-	setup_menuitem (menuitem, 
-			GTK_ICON_SIZE_MENU,
-			gtk_image_new_from_stock (GTK_STOCK_NEW, GTK_ICON_SIZE_MENU),
-			_("_New Panel"),
-			FALSE);
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
-	g_signal_connect (menuitem, "activate",
-			  G_CALLBACK (create_new_panel), 
-			  NULL);
-	gtk_widget_set_sensitive (menuitem, 
-				  panel_profile_id_lists_are_writable ());
-
-	add_menu_separator (menu);
-}
-
-static void
-show_panel_help (GtkWidget *w, gpointer data)
-{
-	panel_show_help (
-		gtk_widget_get_screen (w), "user-guide.xml", "gospanel-5");
-}
-
+  
 GtkWidget *
-create_panel_context_menu (PanelWidget *panel)
+menu_create_action_item (PanelActionButtonType action_type)
 {
-	GtkWidget *retval;
-	GtkWidget *menuitem;
-	char      *gnome_about;
+	GtkWidget *item;
 
-	retval = create_empty_menu ();
+	if (panel_action_get_is_disabled (action_type))
+		return NULL;
 
-	if (!panel_lockdown_get_locked_down ())
-		make_panel_submenu (panel, retval);
+	item = gtk_image_menu_item_new ();
+        setup_menu_item_with_icon (item,
+				   panel_menu_icon_get_size (),
+				   panel_action_get_icon_name (action_type),
+				   NULL,
+				   panel_action_get_text (action_type),
+				   TRUE);
 
-	menuitem = gtk_image_menu_item_new ();
-	setup_menuitem (menuitem,
-			GTK_ICON_SIZE_MENU,
-			gtk_image_new_from_stock (
-				GTK_STOCK_HELP, GTK_ICON_SIZE_MENU),
-				_("_Help"),
-				FALSE);
-	gtk_menu_shell_append (GTK_MENU_SHELL (retval), menuitem);
-	g_signal_connect (menuitem, "activate",
-			  G_CALLBACK (show_panel_help), NULL);
-
-	menuitem = gtk_image_menu_item_new ();
-	setup_menuitem (menuitem,
-			GTK_ICON_SIZE_MENU,
-			gtk_image_new_from_stock (
-				GTK_STOCK_ABOUT, GTK_ICON_SIZE_MENU),
-				_("A_bout Panels"),
-				FALSE);
-	gtk_menu_shell_append (GTK_MENU_SHELL (retval), menuitem);
-	g_signal_connect (menuitem, "activate", G_CALLBACK (show_about_dialog), NULL);
-	
-	gnome_about = g_find_program_in_path ("gnome-about");
-	if (gnome_about) {
-		add_menu_separator (retval);
-
-		menuitem = gtk_image_menu_item_new ();
-		setup_menuitem (menuitem,
-				GTK_ICON_SIZE_MENU,
-				gtk_image_new_from_stock (
-					GTK_STOCK_ABOUT, GTK_ICON_SIZE_MENU),
-					_("About _GNOME"),
-					FALSE);
-		gtk_menu_shell_append (GTK_MENU_SHELL (retval), menuitem);
-		g_signal_connect_data (menuitem, "activate",
-				       G_CALLBACK (about_gnome_cb),
-				       gnome_about, (GClosureNotify) g_free, 0);
-	}
-
-	g_object_set_data (G_OBJECT (retval), "menu_panel", panel);
-
-	return retval;
-}
-
-static void
-append_lock_screen (GtkWidget *menu)
-{
-	GtkWidget  *menuitem;
-
-	if (panel_lockdown_get_disable_lock_screen () ||
-	    !panel_is_program_in_path ("xscreensaver"))
-		return;
-
-	menuitem = gtk_image_menu_item_new ();
-	setup_stock_menu_item (menuitem, panel_menu_icon_get_size (),
-			       panel_action_get_stock_icon (PANEL_ACTION_LOCK),
-			       panel_action_get_text (PANEL_ACTION_LOCK),
-			       TRUE);
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
-	g_signal_connect (menuitem, "activate",
-			  G_CALLBACK (panel_action_lock_screen), NULL);
-	setup_internal_applet_drag (menuitem,
-				    panel_action_get_drag_id (PANEL_ACTION_LOCK));
-	gtk_tooltips_set_tip (panel_tooltips, menuitem,
-			      panel_action_get_tooltip (PANEL_ACTION_LOCK),
+	gtk_tooltips_set_tip (panel_tooltips,
+			      item,
+			      panel_action_get_tooltip (action_type),
 			      NULL);
-}
 
-static void
-append_log_out (GtkWidget *menu)
-{
-	GtkWidget *menuitem;
+	g_signal_connect (item, "activate",
+			  panel_action_get_invoke (action_type), NULL);
+	g_signal_connect (G_OBJECT (item), "button_press_event",
+			  G_CALLBACK (menu_dummy_button_press_event), NULL);
+	setup_internal_applet_drag (item, action_type);
 
-	if (panel_lockdown_get_disable_log_out ())
-		return;
-
-	menuitem = gtk_image_menu_item_new ();
-
-	setup_stock_menu_item (menuitem, panel_menu_icon_get_size (),
-			       panel_action_get_stock_icon (PANEL_ACTION_LOGOUT),
-			       panel_action_get_text (PANEL_ACTION_LOGOUT),
-			       TRUE);
-
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
-	g_signal_connect (menuitem, "activate",
-			  G_CALLBACK (panel_action_logout), NULL);
-	setup_internal_applet_drag (menuitem,
-				    panel_action_get_drag_id (PANEL_ACTION_LOGOUT));
-
-	gtk_tooltips_set_tip (panel_tooltips, menuitem,
-			      panel_action_get_tooltip (PANEL_ACTION_LOGOUT),
-			      NULL);
+	return item;
 }
 
 GtkWidget *
@@ -1743,52 +1525,29 @@ create_main_menu (PanelWidget *panel)
 	g_object_set_data (G_OBJECT (main_menu), "menu_panel", panel);
 
 	menuitem = gtk_image_menu_item_new ();
-	setup_stock_menu_item (menuitem,
-			       panel_menu_icon_get_size (),
-			       PANEL_STOCK_APPLICATIONS,
-			       _("Applications"),
-			       TRUE);
+	setup_menu_item_with_icon (menuitem,
+				   panel_menu_icon_get_size (),
+				   "gnome-applications",
+				   NULL,
+				   _("Applications"),
+				   TRUE);
 	gtk_menu_shell_append (GTK_MENU_SHELL (main_menu), menuitem);
 
-	applications_menu = create_applications_menu ("applications.menu", NULL);
+	applications_menu = create_applications_menu ("applications.menu",
+						      NULL);
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), applications_menu);
 
 	add_menu_separator (main_menu);
 
-	if (!panel_lockdown_get_disable_command_line ()) {
-		menuitem = gtk_image_menu_item_new ();
-		setup_stock_menu_item (menuitem,
-				       panel_menu_icon_get_size (),
-				       panel_action_get_stock_icon (PANEL_ACTION_RUN),
-				       panel_action_get_text (PANEL_ACTION_RUN),
-				       TRUE);
-		g_signal_connect (menuitem, "activate",
-				  G_CALLBACK (panel_action_run_program), NULL);
-		gtk_tooltips_set_tip (panel_tooltips,
-				      menuitem,
-				      panel_action_get_tooltip (PANEL_ACTION_RUN),
-				      NULL);
-		setup_internal_applet_drag (menuitem,
-					    panel_action_get_drag_id (PANEL_ACTION_RUN));
+	menuitem = menu_create_action_item (PANEL_ACTION_RUN);
+	if (menuitem != NULL)
 		gtk_menu_shell_append (GTK_MENU_SHELL (main_menu), menuitem);
-	}
 
 	if (panel_is_program_in_path  ("gnome-search-tool")) {
-		menuitem = gtk_image_menu_item_new ();
-		setup_stock_menu_item (menuitem,
-				       panel_menu_icon_get_size (),
-				       panel_action_get_stock_icon (PANEL_ACTION_SEARCH),
-				       panel_action_get_text (PANEL_ACTION_SEARCH),
-				       TRUE);
-		g_signal_connect (menuitem, "activate",
-				  G_CALLBACK (panel_action_search), NULL);
-		gtk_tooltips_set_tip (panel_tooltips,
-				      menuitem,
-				      panel_action_get_tooltip (PANEL_ACTION_SEARCH),
-				      NULL);
-		setup_internal_applet_drag (menuitem,
-					    panel_action_get_drag_id (PANEL_ACTION_SEARCH));
-		gtk_menu_shell_append (GTK_MENU_SHELL (main_menu), menuitem);
+		menuitem = menu_create_action_item (PANEL_ACTION_SEARCH);
+		if (menuitem != NULL)
+			gtk_menu_shell_append (GTK_MENU_SHELL (main_menu),
+					       menuitem);
 	}
 
 	panel_recent_append_documents_menu (main_menu);
@@ -1796,27 +1555,24 @@ create_main_menu (PanelWidget *panel)
 	add_menu_separator (main_menu);
 
 	if (panel_is_program_in_path ("gnome-screenshot")) {
-		menuitem = gtk_image_menu_item_new ();
-		setup_stock_menu_item (menuitem,
-				       panel_menu_icon_get_size (),
-				       panel_action_get_stock_icon (PANEL_ACTION_SCREENSHOT),
-				       panel_action_get_text (PANEL_ACTION_SCREENSHOT),
-				       TRUE);
-		g_signal_connect (menuitem, "activate",
-				  G_CALLBACK (panel_action_screenshot), NULL);
-		gtk_tooltips_set_tip (panel_tooltips,
-				      menuitem,
-				      panel_action_get_tooltip (PANEL_ACTION_SCREENSHOT),
-				      NULL);
-		setup_internal_applet_drag (menuitem,
-					    panel_action_get_drag_id (PANEL_ACTION_SCREENSHOT));
-		gtk_menu_shell_append (GTK_MENU_SHELL (main_menu), menuitem);
+		menuitem = menu_create_action_item (PANEL_ACTION_SCREENSHOT);
+		if (menuitem != NULL)
+			gtk_menu_shell_append (GTK_MENU_SHELL (main_menu),
+					       menuitem);
 
 		add_menu_separator (main_menu);
 	}
 
-	append_lock_screen (main_menu);
-	append_log_out (main_menu);
+	if (panel_is_program_in_path  ("xscreensaver")) {
+		menuitem = menu_create_action_item (PANEL_ACTION_LOCK);
+		if (menuitem != NULL)
+			gtk_menu_shell_append (GTK_MENU_SHELL (main_menu),
+					       menuitem);
+	}
+
+	menuitem = menu_create_action_item (PANEL_ACTION_LOGOUT);
+	if (menuitem != NULL)
+		gtk_menu_shell_append (GTK_MENU_SHELL (main_menu), menuitem);
 
 	return main_menu;
 }
@@ -1836,15 +1592,19 @@ find_in_load_list (GtkWidget *image)
 static void
 image_menu_shown (GtkWidget *image, gpointer data)
 {
-	IconToLoad *icon = data;
+	IconToLoad *new_icon;
+	IconToLoad *icon;
+	
+	icon = (IconToLoad *) data;
 
 	/* if we've already handled this */
 	if (gtk_image_get_storage_type (GTK_IMAGE (image)) != GTK_IMAGE_EMPTY)
 		return;
 
 	if (find_in_load_list (image) == NULL) {
-		icons_to_load = g_list_append (icons_to_load,
-					       icon_to_load_copy (icon));
+		new_icon = icon_to_load_copy (icon);
+		new_icon->icon_theme = gtk_icon_theme_get_for_screen (gtk_widget_get_screen (image));
+		icons_to_load = g_list_append (icons_to_load, new_icon);
 	}
 	if (load_icons_id == 0)
 		load_icons_id = g_idle_add (load_icons_handler, NULL);
@@ -1916,14 +1676,14 @@ panel_menu_key_press_handler (GtkWidget   *widget,
 	    (event->state & gtk_accelerator_get_default_mod_mask ()) == GDK_SHIFT_MASK)) {
 		GtkMenuShell *menu_shell = GTK_MENU_SHELL (widget);
 
-		retval = TRUE;
 		if (menu_shell->active_menu_item &&
 		    GTK_MENU_ITEM (menu_shell->active_menu_item)->submenu == NULL) {
 			GdkEventButton bevent;
 
 			bevent.button = 3;
 			bevent.time = GDK_CURRENT_TIME;
-			show_item_menu (menu_shell->active_menu_item, &bevent);
+			retval = show_item_menu (menu_shell->active_menu_item,
+						 &bevent);
 		}
 		
 	}
