@@ -33,7 +33,6 @@
 #include "panel.h"
 #include "drawer.h"
 #include "panel-applet-frame.h"
-#include "panel-stock-icons.h"
 #include "panel-action-button.h"
 #include "panel-menu-bar.h"
 #include "panel-toplevel.h"
@@ -59,6 +58,7 @@ typedef struct {
 
 	GSList       *applet_list;
 	GSList       *application_list;
+	GSList       *settings_list;
 
 	guint         name_notify;
 
@@ -80,14 +80,14 @@ typedef enum {
 
 typedef struct {
 	PanelAddtoItemType     type;
-	const char            *name;
-	const char            *description;
-	const char            *icon;
-	const char            *stock_icon;
+	char                  *name;
+	char                  *description;
+	char                  *icon;
 	PanelActionButtonType  action_type;
-	const char            *launcher_path;
-	const char            *menu_path;
-	const char            *iid;
+	char                  *launcher_path;
+	char                  *menu_filename;
+	char                  *menu_path;
+	char                  *iid;
 	gboolean               static_data;
 } PanelAddtoItemInfo;
 
@@ -101,22 +101,22 @@ static PanelAddtoItemInfo special_addto_items [] = {
 	{ PANEL_ADDTO_LAUNCHER_NEW,
 	  N_("Custom Application Launcher"),
 	  N_("Create a new launcher"),
-	  NULL,
-	  PANEL_STOCK_LAUNCHER,
+	  "launcher-program",
 	  PANEL_ACTION_NONE,
 	  NULL,
 	  NULL,
+	  NULL,
 	  "LAUNCHER:ASK",
 	  TRUE },
 
 	{ PANEL_ADDTO_LAUNCHER_MENU,
 	  N_("Application Launcher..."),
 	  N_("Launch a program that is already in the GNOME menu"),
-	  NULL,
-	  PANEL_STOCK_LAUNCHER,
+	  "launcher-program",
 	  PANEL_ACTION_NONE,
 	  NULL,
 	  NULL,
+	  NULL,
 	  "LAUNCHER:MENU",
 	  TRUE }
 
@@ -127,33 +127,33 @@ static PanelAddtoItemInfo internal_addto_items [] = {
 	{ PANEL_ADDTO_MENU,
 	  N_("Main Menu"),
 	  N_("The main GNOME menu"),
-	  NULL,
-	  PANEL_STOCK_MAIN_MENU,
+	  PANEL_MAIN_MENU_ICON,
 	  PANEL_ACTION_NONE,
 	  NULL,
 	  NULL,
+	  NULL,
 	  "MENU:MAIN",
 	  TRUE },
 
 	{ PANEL_ADDTO_MENUBAR,
 	  N_("Menu Bar"),
 	  N_("A custom menu bar"),
-	  NULL,
-	  PANEL_STOCK_GNOME_LOGO,
+	  PANEL_GNOME_LOGO_ICON,
 	  PANEL_ACTION_NONE,
 	  NULL,
 	  NULL,
+	  NULL,
 	  "MENUBAR:NEW",
 	  TRUE },
 
 	{ PANEL_ADDTO_DRAWER,
 	  N_("Drawer"),
 	  N_("A pop out drawer to store other items in"),
-	  NULL,
-	  PANEL_STOCK_DRAWER,
+	  PANEL_DRAWER_ICON,
 	  PANEL_ACTION_NONE,
 	  NULL,
 	  NULL,
+	  NULL,
 	  "DRAWER:NEW",
 	  TRUE }
 };
@@ -192,7 +192,7 @@ panel_addto_applet_info_sort_func (PanelAddtoItemInfo *a,
 }
 
 static GSList *
-panel_addto_append_internal_applets (GSList *list)
+panel_addto_prepend_internal_applets (GSList *list)
 {
 	static gboolean translated = FALSE;
 	int             i;
@@ -203,7 +203,7 @@ panel_addto_append_internal_applets (GSList *list)
 			internal_addto_items [i].description = _(internal_addto_items [i].description);
 		}
 
-                list = g_slist_append (list, &internal_addto_items [i]);
+                list = g_slist_prepend (list, &internal_addto_items [i]);
         }
 
 	translated = TRUE;
@@ -219,11 +219,11 @@ panel_addto_append_internal_applets (GSList *list)
 		info->action_type = i;
 		info->name        = (char *) panel_action_get_text (i);
 		info->description = (char *) panel_action_get_tooltip (i);
-		info->stock_icon  = (char *) panel_action_get_stock_icon (i);
+		info->icon        = (char *) panel_action_get_icon_name (i);
 		info->iid         = (char *) panel_action_get_drag_id (i);
-		info->static_data = FALSE;
+		info->static_data = TRUE;
 
-                list = g_slist_append (list, info);
+                list = g_slist_prepend (list, info);
 	}
 
         return list;
@@ -238,7 +238,7 @@ panel_addto_make_text (const char *name,
 
 	real_name = name ? name : _("(empty)");
 
-	if (desc != NULL && desc[0] != '\0') {
+	if (!string_empty (desc)) {
 		result = g_markup_printf_escaped ("<span size=\"larger\" weight=\"bold\">%s</span>\n%s",
 						  real_name, desc);
 	} else {
@@ -253,7 +253,6 @@ static GdkPixbuf *
 panel_addto_make_pixbuf (const char *filename,
 			 GtkIconSize size)
 {
-	char *file;
 	GdkPixbuf *pb, *newpb;
 	int width, height;
 	int desired_width, desired_height;
@@ -261,14 +260,11 @@ panel_addto_make_pixbuf (const char *filename,
 	if (!gtk_icon_size_lookup (size, &desired_width, &desired_height))
 		return NULL;
 
-	file = gnome_desktop_item_find_icon (panel_icon_theme,
-					     filename, desired_height, 0);
-
-	if (file == NULL)
+	pb = panel_load_icon (gtk_icon_theme_get_default (), filename,
+			      desired_height, NULL);
+	if (!pb)
 		return NULL;
 
-	pb = gdk_pixbuf_new_from_file_at_size (file, desired_width,
-					       desired_height, NULL);
 	width = gdk_pixbuf_get_width (pb);
 	height = gdk_pixbuf_get_height (pb);
 
@@ -288,7 +284,6 @@ panel_addto_make_pixbuf (const char *filename,
 		pb = newpb;
 	}
 
-	g_free (file);
 	return pb;
 }
 
@@ -305,6 +300,30 @@ panel_addto_drag_data_get_cb (GtkWidget        *widget,
 				strlen (string));
 }
 
+static void
+panel_addto_drag_begin_cb (GtkWidget      *widget,
+			   GdkDragContext *context,
+			   gpointer        data)
+{
+	GtkTreeModel *model;
+	GtkTreePath  *path;
+	GtkTreeIter   iter;
+	GdkPixbuf    *pixbuf;
+
+	model = gtk_tree_view_get_model (GTK_TREE_VIEW (widget));
+	   
+	gtk_tree_view_get_cursor (GTK_TREE_VIEW (widget), &path, NULL);
+	gtk_tree_model_get_iter (model, &iter, path);
+	gtk_tree_path_free (path);
+
+	gtk_tree_model_get (model, &iter,
+	                    COLUMN_ICON, &pixbuf,
+	                    -1);
+
+	gtk_drag_set_icon_pixbuf (context, pixbuf, 0, 0);
+	g_object_unref (pixbuf);
+}
+
 static void
 panel_addto_setup_drag (GtkTreeView          *tree_view,
 			const GtkTargetEntry *target,
@@ -322,6 +341,9 @@ panel_addto_setup_drag (GtkTreeView          *tree_view,
 			       g_strdup (text),
 			       (GClosureNotify) g_free,
 			       0 /* connect_flags */);
+	g_signal_connect_after (G_OBJECT (tree_view), "drag-begin",
+	                        G_CALLBACK (panel_addto_drag_begin_cb),
+	                        NULL);
 }
 
 static void
@@ -379,6 +401,7 @@ panel_addto_query_applets (GSList *list)
 			   BONOBO_EX_REPOID (&env));
 
 		CORBA_exception_free (&env);
+		CORBA_free (applet_list);
 
 		return NULL;
 	}
@@ -416,16 +439,17 @@ panel_addto_query_applets (GSList *list)
 
 		applet = g_new0 (PanelAddtoItemInfo, 1);
 		applet->type = PANEL_ADDTO_APPLET;
-		applet->name = name;
-		applet->description = description;
-		applet->icon = icon;
-		applet->iid = info->iid;
+		applet->name = g_strdup (name);
+		applet->description = g_strdup (description);
+		applet->icon = g_strdup (icon);
+		applet->iid = g_strdup (info->iid);
 		applet->static_data = FALSE;
 
-		list = g_slist_append (list, applet);
+		list = g_slist_prepend (list, applet);
 	}
 
 	g_slist_free (langs_gslist);
+	CORBA_free (applet_list);
 
 	return list;
 }
@@ -448,14 +472,11 @@ panel_addto_append_item (PanelAddtoDialog *dialog,
 				    COLUMN_SEARCH, NULL,
 				    -1);
 	} else {
+		pixbuf = NULL;
+
 		if (applet->icon != NULL) {
 			pixbuf = panel_addto_make_pixbuf (applet->icon,
 							  GTK_ICON_SIZE_DIALOG);
-		} else {
-			pixbuf = gtk_widget_render_icon (GTK_WIDGET (dialog->panel_widget),
-							 applet->stock_icon,
-							 GTK_ICON_SIZE_DIALOG,
-							 NULL);
 		}
 
 		gtk_list_store_append (model, &iter);
@@ -500,7 +521,7 @@ panel_addto_make_applet_model (PanelAddtoDialog *dialog)
 
 	if (panel_profile_id_lists_are_writable ()) {
 		dialog->applet_list = panel_addto_query_applets (dialog->applet_list);
-		dialog->applet_list = panel_addto_append_internal_applets (dialog->applet_list);
+		dialog->applet_list = panel_addto_prepend_internal_applets (dialog->applet_list);
 	}
 
 	dialog->applet_list = g_slist_sort (dialog->applet_list,
@@ -526,7 +547,8 @@ panel_addto_make_applet_model (PanelAddtoDialog *dialog)
 
 static void
 panel_addto_make_application_list (GSList            **parent_list,
-				   MenuTreeDirectory  *directory)
+				   MenuTreeDirectory  *directory,
+				   const char         *filename)
 {
 	GSList *subdirs;
 	GSList *entries;
@@ -543,7 +565,9 @@ panel_addto_make_application_list (GSList            **parent_list,
 		data->item_info.name        = g_strdup (menu_tree_directory_get_name (subdir));
 		data->item_info.description = g_strdup (menu_tree_directory_get_comment (subdir));
 		data->item_info.icon        = g_strdup (menu_tree_directory_get_icon (subdir));
+		data->item_info.menu_filename = g_strdup (filename);
 		data->item_info.menu_path   = menu_tree_directory_make_path (subdir, NULL);
+		data->item_info.static_data = FALSE;
 
 		/* We should set the iid here to something and do
 		 * iid = g_strdup_printf ("MENU:%s", tfr->name)
@@ -553,9 +577,10 @@ panel_addto_make_application_list (GSList            **parent_list,
 		 * So the iid is built when we select the row.
 		 */
 
-		*parent_list = g_slist_append (*parent_list, data);
+		*parent_list = g_slist_prepend (*parent_list, data);
 
-		panel_addto_make_application_list (&data->children, subdir);
+		panel_addto_make_application_list (&data->children, subdir,
+						   filename);
 
 		menu_tree_directory_unref (subdir);
 	}
@@ -573,12 +598,15 @@ panel_addto_make_application_list (GSList            **parent_list,
 		data->item_info.description   = g_strdup (menu_tree_entry_get_comment (entry));
 		data->item_info.icon          = g_strdup (menu_tree_entry_get_icon (entry));
 		data->item_info.launcher_path = g_strdup (menu_tree_entry_get_desktop_file_path (entry));
+		data->item_info.static_data = FALSE;
 
-		*parent_list = g_slist_append (*parent_list, data);
+		*parent_list = g_slist_prepend (*parent_list, data);
 
 		menu_tree_entry_unref (entry);
 	}
 	g_slist_free (entries);
+
+	*parent_list = g_slist_reverse (*parent_list);
 }
 
 static void
@@ -635,7 +663,8 @@ panel_addto_make_application_model (PanelAddtoDialog *dialog)
 	tree = menu_tree_lookup ("applications.menu");
 
 	if ((root = menu_tree_get_root_directory (tree))) {
-		panel_addto_make_application_list (&dialog->application_list, root);
+		panel_addto_make_application_list (&dialog->application_list,
+						   root, "applications.menu");
 		panel_addto_populate_application_model (store, NULL, dialog->application_list);
 
 		menu_tree_directory_unref (root);
@@ -643,6 +672,29 @@ panel_addto_make_application_model (PanelAddtoDialog *dialog)
 
 	menu_tree_unref (tree);
 
+	tree = menu_tree_lookup ("settings.menu");
+
+	if ((root = menu_tree_get_root_directory (tree))) {
+		GtkTreeIter iter;
+
+		gtk_tree_store_append (store, &iter, NULL);
+		gtk_tree_store_set (store, &iter,
+				    COLUMN_ICON, NULL,
+				    COLUMN_TEXT, NULL,
+				    COLUMN_DATA, NULL,
+				    COLUMN_SEARCH, NULL,
+				    -1);
+
+		panel_addto_make_application_list (&dialog->settings_list,
+						   root, "settings.menu");
+		panel_addto_populate_application_model (store, NULL,
+							dialog->settings_list);
+
+		menu_tree_directory_unref (root);
+	}
+
+	menu_tree_unref (tree);
+
 	return GTK_TREE_MODEL (store);
 }
 
@@ -683,6 +735,7 @@ panel_addto_add_item (PanelAddtoDialog   *dialog,
 	case PANEL_ADDTO_MENU:
 		panel_menu_button_create (dialog->panel_widget->toplevel,
 					  dialog->insertion_position,
+					  item_info->menu_filename,
 					  item_info->menu_path,
 					  item_info->menu_path != NULL,
 					  item_info->name);
@@ -782,6 +835,41 @@ panel_addto_present_applets (PanelAddtoDialog *dialog)
 	gtk_widget_set_sensitive (dialog->back_button, FALSE);
 }
 
+static void
+panel_addto_dialog_free_item_info (PanelAddtoItemInfo *item_info)
+{
+	if (item_info == NULL || item_info->static_data)
+		return;
+
+	if (item_info->name != NULL)
+		g_free (item_info->name);
+	item_info->name = NULL;
+
+	if (item_info->description != NULL)
+		g_free (item_info->description);
+	item_info->description = NULL;
+
+	if (item_info->icon != NULL)
+		g_free (item_info->icon);
+	item_info->icon = NULL;
+
+	if (item_info->iid != NULL)
+		g_free (item_info->iid);
+	item_info->iid = NULL;
+
+	if (item_info->launcher_path != NULL)
+		g_free (item_info->launcher_path);
+	item_info->launcher_path = NULL;
+
+	if (item_info->menu_filename != NULL)
+		g_free (item_info->menu_filename);
+	item_info->menu_filename = NULL;
+
+	if (item_info->menu_path != NULL)
+		g_free (item_info->menu_path);
+	item_info->menu_path = NULL;
+}
+
 static void
 panel_addto_dialog_free_application_list (GSList *application_list)
 {
@@ -796,6 +884,7 @@ panel_addto_dialog_free_application_list (GSList *application_list)
 		if (data->children) {
 			panel_addto_dialog_free_application_list (data->children);
 		}
+		panel_addto_dialog_free_item_info (&data->item_info);
 		g_free (data);
 	}
 	g_slist_free (application_list);
@@ -822,12 +911,14 @@ panel_addto_dialog_free (PanelAddtoDialog *dialog)
 
 		applet = (PanelAddtoItemInfo *) item->data;
 		if (!applet->static_data) {
+			panel_addto_dialog_free_item_info (applet);
 			g_free (applet);
 		}
 	}
 	g_slist_free (dialog->applet_list);
 
 	panel_addto_dialog_free_application_list (dialog->application_list);
+	panel_addto_dialog_free_application_list (dialog->settings_list);
 
 	if (dialog->menu_tree)
 		menu_tree_unref (dialog->menu_tree);
@@ -841,28 +932,29 @@ panel_addto_name_change (PanelAddtoDialog *dialog,
 			 const char       *name)
 {
 	char *title;
-	char *markup_title;
+	char *label;
 
-	if (name) {
-		title = g_strdup_printf (_("Add to %s"), name);
+	label = NULL;
+
+	if (!string_empty (name))
+		label = g_strdup_printf (_("Select an _item to add to \"%s\":"),
+					 name);
+
+	if (panel_toplevel_get_is_attached (dialog->panel_widget->toplevel)) {
+		title = g_strdup_printf (_("Add to Drawer"));
+		if (label == NULL)
+			label = g_strdup (_("Select an _item to add to the drawer:"));
 	} else {
-		title = g_strdup_printf (_("Add to the panel"));
+		title = g_strdup_printf (_("Add to Panel"));
+		if (label == NULL)
+			label = g_strdup (_("Select an _item to add to the panel:"));
 	}
+
 	gtk_window_set_title (GTK_WINDOW (dialog->addto_dialog), title);
 	g_free (title);
 
-	if (name) {
-		title = g_strdup_printf (_("Select an _item to add to %s:"),
-					 name);
-	} else {
-		title = g_strdup (_("Select an _item to add to the panel:"));
-	}
-	markup_title = g_strdup_printf ("<span weight=\"bold\">%s</span>",
-					title);
-	g_free (title);
-	gtk_label_set_markup_with_mnemonic (GTK_LABEL (dialog->label),
-					    markup_title);
-	g_free (markup_title);
+	gtk_label_set_text_with_mnemonic (GTK_LABEL (dialog->label), label);
+	g_free (label);
 }
 
 static void
@@ -915,15 +1007,12 @@ panel_addto_selection_changed (GtkTreeSelection *selection,
 	if (data->type == PANEL_ADDTO_LAUNCHER_MENU) {
 		gtk_button_set_label (GTK_BUTTON (dialog->add_button),
 				      GTK_STOCK_GO_FORWARD);
-		gtk_button_set_use_stock (GTK_BUTTON (dialog->add_button),
-					  TRUE);
 	} else {
-		//FIXME
 		gtk_button_set_label (GTK_BUTTON (dialog->add_button),
 				      GTK_STOCK_ADD);
-		gtk_button_set_use_stock (GTK_BUTTON (dialog->add_button),
-					  TRUE);
 	}
+	gtk_button_set_use_stock (GTK_BUTTON (dialog->add_button),
+				  TRUE);
 
 	/* only allow dragging applets if we can add applets */
 	if (panel_profile_id_lists_are_writable ()) {
@@ -942,7 +1031,8 @@ panel_addto_selection_changed (GtkTreeSelection *selection,
 		case PANEL_ADDTO_MENU:
 			/* build the iid for menus other than the main menu */
 			if (data->iid == NULL) {
-				iid = g_strdup_printf ("MENU:%s",
+				iid = g_strdup_printf ("MENU:%s/%s",
+						       data->menu_filename,
 						       data->menu_path);
 			} else {
 				iid = g_strdup (data->iid);
@@ -1016,12 +1106,12 @@ panel_addto_dialog_new (PanelWidget *panel_widget)
 	dialog->addto_dialog = gtk_dialog_new ();
 	gtk_dialog_add_button (GTK_DIALOG (dialog->addto_dialog),
 			       GTK_STOCK_HELP, GTK_RESPONSE_HELP);
-	gtk_dialog_add_button (GTK_DIALOG (dialog->addto_dialog),
-			       GTK_STOCK_CANCEL,
-			       GTK_RESPONSE_CANCEL);
 	dialog->back_button = gtk_dialog_add_button (GTK_DIALOG (dialog->addto_dialog),
 						     GTK_STOCK_GO_BACK,
 						     PANEL_ADDTO_RESPONSE_BACK);
+	gtk_dialog_add_button (GTK_DIALOG (dialog->addto_dialog),
+			       GTK_STOCK_CANCEL,
+			       GTK_RESPONSE_CANCEL);
 	dialog->add_button = gtk_dialog_add_button (GTK_DIALOG (dialog->addto_dialog),
 						     GTK_STOCK_ADD,
 						     PANEL_ADDTO_RESPONSE_ADD);
