@@ -39,6 +39,7 @@ GList *panel_g_list_resort_item   (GList        *list,
 
 GtkWidget      *panel_error_dialog      (GdkScreen  *screen,
 					 const char *class,
+					 gboolean    auto_destroy,
 					 const char *primary_format,
 					 const char *secondary_format,
 					 ...);
@@ -55,12 +56,20 @@ gboolean	panel_uri_exists	(const char *uri);
 
 void            panel_lock_screen       (GdkScreen *screen);
 
+char *          panel_find_icon         (GtkIconTheme  *icon_theme,
+					 const char    *icon_name,
+					 gint           size);
+GdkPixbuf *     panel_load_icon         (GtkIconTheme  *icon_theme,
+					 const char    *icon_name,
+					 gint           size,
+					 char         **error_msg);
 GdkPixbuf *	missing_pixbuf		(int size);
 
 char *panel_make_full_path   (const char *dir,
 			      const char *filename);
 char *panel_make_unique_uri (const char *dir,
 			     const char *suffix);
+char *panel_lookup_in_data_dirs (const char *basename);
 
 G_END_DECLS
 
